@@ -29,16 +29,25 @@ module Async
 		def initialize(io, reactor = nil)
 			@io = io
 			
-			@reactor = reactor || Task.current.reactor
+			@reactor = reactor
 			@monitor = nil
 		end
 		
 		# The underlying native `io`.
 		attr :io
 		
-		# The reactor this wrapper is associated with.
+		# The reactor this wrapper is associated with, if any.
 		attr :reactor
 		
+		# Bind this wrapper to a different reactor. Assign nil to convert to an unbound wrapper (can be used from any reactor/task but with slightly increased overhead.)
+		# Binding to a reactor is purely a performance consideration. Generally, I don't like APIs that exist only due to optimisations. This is borderline, so consider this functionality semi-private.
+		def reactor= reactor
+			@monitor.close if @monitor
+			
+			@reactor = reactor
+			@monitor = nil
+		end
+		
 		# Wait for the io to become readable.
 		def wait_readable(duration = nil)
 			wait_any(:r, duration)
@@ -53,50 +62,46 @@ module Async
 		# @param interests [:r | :w | :rw] what events to wait for.
 		# @param duration [Float] timeout after the given duration if not `nil`.
 		def wait_any(interests = :rw, duration = nil)
-			monitor(interests, duration)
+			# There is value in caching this monitor - if you can reuse it, you will get about 2x the throughput, because you avoid calling Reactor#register and Monitor#close for every call. That being said, by caching it, you also introduce lifetime issues. I'm going to accept this overhead into the wrapper design because it's pretty convenient, but if you want faster IO, take a look at the performance spec which compares this method with a more direct alternative.
+			if @reactor
+				unless @monitor
+					@monitor = @reactor.register(@io, interests)
+				else
+					@monitor.interests = interests
+					@monitor.value = Fiber.current
+				end
+				
+				begin
+					wait_for(@reactor, @monitor, duration)
+				ensure
+					@monitor.remove_interest(@monitor.interests)
+					@monitor.value = nil
+				end
+			else
+				reactor = Task.current.reactor
+				monitor = reactor.register(@io, interests)
+				
+				begin
+					wait_for(reactor, monitor, duration)
+				ensure
+					monitor.close
+				end
+			end
 		end
 		
-		# Close the monitor.
+		# Close the io and monitor.
 		def close
-			close_monitor
+			@monitor.close if @monitor
 			
-			@io.close if @io
+			@io.close
 		end
 		
 		private
 		
-		def close_monitor
-			if @monitor
-				@monitor.close
-				@monitor = nil
-			end
-		end
-		
-		if ::NIO::VERSION >= "2.0"
-			def clear_monitor
-				if @monitor
-					# Alas, @monitor.interests = nil does not yet work.
-					@monitor.value = nil
-					@monitor.remove_interest(@monitor.interests)
-				end
-			end
-		else
-			alias clear_monitor close_monitor
-		end
-		
-		# Monitor the io for the given events
-		def monitor(interests, duration = nil)
-			unless @monitor
-				@monitor = @reactor.register(@io, interests)
-			else
-				@monitor.interests = interests
-			end
-			
-			@monitor.value = Fiber.current
-			
+		def wait_for(reactor, monitor, duration)
 			# If the user requested an explicit timeout for this operation:
 			if duration
-				@reactor.timeout(duration) do
+				reactor.timeout(duration) do
 					Task.yield
 				end
 			else
@@ -104,8 +109,6 @@ module Async
 			end
 			
 			return true
-		ensure
-			clear_monitor
 		end
 	end
 end
